! ===========================================================================================================
! character function write_nctraj.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes:
!
! This function creates the traj nc file.
! ===========================================================================================================
! Function returns 'y' for successful file creation, "F" for failure
! ===========================================================================================================

character function write_nctraj(nctraj)

use communication                       ! module
use data_arrays                         ! module
use netcdf_arrays			! module
use nctraj_arrays			! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(len=128) , intent( in ) :: nctraj

integer :: ncid;
 
character(len=12),parameter :: sub = 'write_nctraj'
character(len=4),parameter :: nctype = 'traj'
character(len=5),parameter :: ncjob = 'WRITE'; character(len=4) :: nctup;  ! What job are we doing, nctupe_uppercase
integer :: ret = 0;


! ===========================================================================================================
! executable statements

! Which file is being written, and wat is the filmsg index?
filmsg(l:l) = 'R'; call wrtflm(sub,'Initialize to Running')

call set_uppercase(nctype,nctup)
gloc = 3000; ! gloc - global location of the nc call
gnctype = nctype; ! For calls to cke

if(DEBUG)then; call dbg_ncbeg(sub,nctup,ncjob); endif


! 'N_PARAM',                 N_PARAM_len
! 'N_CYCLE',                 N_CYCLE_len
! 'N_MEASUREMENT',           N_MEASUREMENT_len             ! UNLIMITED for nctraj
! 'N_HISTORY',               N_HISTORY_len_traj


ret=create_ncfile(sub,nctype,nctraj,ncid); ! This also defines the dimensins
!write(msg,'("write NCTRAJ: ",a)')trim(nctraj); call wrtstd(sub,msg); ! STD
write(msg,'("write NCTRAJ: ",a,":",a,":",a,":",i5)')trim(nctraj),sub,nctype,ncid; call wrtstd(sub,msg); ! STD
write(msg,'("write NCTRAJ: ",a, i1)')trim(nctraj),ncid; call wrtstd(sub,msg); ! STD

! Did the file fail to auccessfully open? - either out_open=F, or filmsg(l:l) has changed from  'R'
if(.not. out_open)then; write(msg,'(a,"out_opet=F, FILMSG: ",a)')HEDF,filmsg(l:l); call wrttrc(sub,msg); return; endif
if(filmsg(l:l) /= 'R')then; write(msg,'(a,"out_opet=T, FILMSG no R: ",a)')HEDF,filmsg(l:l); call wrttrc(sub,msg); return; endif

write(msg,'(a,"output file opened: ",a)')WRTFILE,trim(nctraj); call wrtlog(sub,msg);

if(DBLEV==65)then; write(DBG,3001)WRTFILE; call close_ncfile(sub,ncid); call direct_exit(sub,0,'LEV65'); endif
3001 format("LEV=65 Exit after nc file creation for ",a);

!if(DEBUG)
 write(msg,*)'Calling def_common_vars'; call wrttrc(sub,msg);
call def_common_vars(sub,nctype,ncid); 
!if(DEBUG)
 write(msg,*)'Calling def_common_vars done'; call wrttrc(sub,msg);

!if(DEBUG) 
write(msg,*)'Calling def_nctraj_vars'; call wrttrc(sub,msg);
call def_nctraj_vars(sub,nctype,ncid);
!if(DEBUG)
 write(msg,*)'Calling def_nctraj_vars done'; call wrttrc(sub,msg);

!if(DEBUG) 
write(msg,*)'Calling end_define_mode'; call wrttrc(sub,msg);
call end_define_mode(sub,ncid);
!if(DEBUG) 
write(msg,*)'Calling end_define_mode done'; call wrttrc(sub,msg);

!if(DEBUG) 
write(msg,*)'Calling put_common_data'; call wrttrc(sub,msg);
call put_common_data(sub,nctype,ncid);
!if(DEBUG)
 write(msg,*)'Calling put_common_data done'; call wrttrc(sub,msg);

!if(DEBUG) 
write(msg,*)'Calling put_nctraj_data'; call wrttrc(sub,msg);
call put_nctraj_data(sub,nctype,ncid);
!if(DEBUG) 
write(msg,*)'Calling put_nctraj_data done'; call wrttrc(sub,msg);

!if(DEBUG) 
write(msg,*)'Calling close_ncfile'; call wrttrc(sub,msg);
call close_ncfile(sub,ncid);
!if(DEBUG)
 write(msg,*)'Calling close_ncfile'; call wrttrc(sub,msg);

! See if an error occurred in an array module
if(filmsg(l:l) == 'R')then;
    filmsg(l:l) = 'y'; call wrtflm(sub,'Normal exit, return y for SUCCESS');
elseif((filmsg(l:l) == 'f') .or. (filmsg(l:l) == 'F'))then
    if(DEBUG) write(msg,'(a,a1,x,a8,x,a)')HEDF,'R',filmsg,'ERROR exit, return F or f'; call wrtdbg(sub,msg);
else
    write(msg,'(a,a1,x,a8,x,a)')HEDF,filmsg(l:l),'W Non-standard file result'; call wrtlog(sub,msg);
endif

write(msg,'(a,"output file closed: ",a)')WRTFILE,trim(nctraj); call wrtlog(sub,msg);

write_nctraj = filmsg(l:l)
 
if(DEBUG)then; call dbg_ncret(sub,nctup,ncjob,write_nctraj); endif
if(DEBUG)then; call dbg_ncend(sub,nctup,ncjob); endif

return          ! Normal return
! ===========================================================================================================
end function write_nctraj
! ===========================================================================================================
