! ===========================================================================================================
! integer function prepare_launch_config.f90
! 
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20180724 VH     Improve warnings for config value problems
!                 Finalize cyc<6 warning/lognote
! 20180422 VH     Mismatch between launch configs & and cyc 000/001 now issued for cyc<6. General WARNING revisions.
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
! Function to read config parameters for netcdf files while passing parameters from qc files
!
! uses: qc_config_params_lst, nc_config_params_lst, cnt_config_table_param    (from read_configtable.f90)
!       npqc, pqcline                                                         (from read_pqc.f90)
! ===========================================================================================================

integer function prepare_launch_config(launch_config_source_indicator)

use communication                       ! module
use data_arrays                         ! module
use ncmeta_arrays                       ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

integer,intent(in) :: launch_config_source_indicator  ! 0 Means Cycle 0 or 1, -1 means Ascii meta

integer :: i_cycle0_param, i_met =0, i_table, i_launch_config
character(len=80)  :: str_meta_tmp2;
character(len=256) :: str_meta_tmp1;
character(len=128) :: str_meta_tmp3;
character(len=5)   :: str_meta_tmp4;
character(len=128) :: str_pqc_tmp
character(len=256) :: str_pqc_tmp1;
character(len=80)  :: str_pqc_tmp2;
character(len=128) :: str_pqc_tmp3; 
double precision   :: config_val_tmp
      
logical :: is_valid_config, config_param_name_exists = .false.

integer :: result = 0; ! number of table values read
character(len=16),parameter :: sub = 'prep_launch_cnfg'
character(len=15),parameter :: typ = 'NCMETA PRLCNFG'
character(len=128) :: fil ! shorthand for filename
integer :: conf_sampl_match_cnt, cycnum=-1;
! ===========================================================================================================
! executable statements

if(DBLEV >90)then
   write(msg,'("==== ",a," BEG ==== launch_config_source_indicator: ",i3," ===============")')typ,launch_config_source_indicator;
   call wrtdbg(sub,msg);
endif

prepare_launch_config = SUCCESS 
     
     
      if(launch_config_source_indicator == -1)then  !To extract details from Ascii meta file
          if(trans_type == 'ARGOS')then ! Add the nominal cycle count as CONFIG from meta file
                 n_combined_launch_config_param = n_combined_launch_config_param + 1
                  combined_launch_config_param_name(n_combined_launch_config_param,1)=&
                 'CONFIG_ParkAndProfileCycleCounter_COUNT'
                 combined_launch_config_param_name(n_combined_launch_config_param,2) &
                 = 'META'  ! To indicate config is from Ascii META file
                 if(nominal_prof_ncyc1+nominal_prof_ncyc2 <=50)then
                    combined_launch_config_param_value(n_combined_launch_config_param)= &
                    DBLE(nominal_prof_ncyc1+nominal_prof_ncyc2)
                 endif
          endif
         do i_met = 1,nmet
                 is_valid_config = .true.
	         str_meta_tmp1 = trim(metline(i_met)); 
	         str_meta_tmp2=trim(str_meta_tmp1(1:40));
	         call to_upper(str_meta_tmp2);
                 str_meta_tmp3=trim(str_meta_tmp1(41:));
		 
		 do i_table=1,cnt_config_table_skip_param
		   if (trim(str_meta_tmp2).eq.trim(qc_config_skip_lst(i_table))) then
                       is_valid_config=.false.;
		       exit;  ! exit the loop, not the program
                   endif
		 enddo 
		 
		 if(is_valid_config)then
	            do i_table=1,cnt_config_table_param
		       if(trim(str_meta_tmp2) .eq. trim(qc_config_params_lst(i_table)))then
                          if(trans_type == 'IRIDIUM' .and. trim(nc_config_params_lst(i_table))== &
                          'CONFIG_TransmissionRepetitionPeriod_seconds')exit !Note from Claudia: this param is not needed for "Iridium"
		          n_combined_launch_config_param = n_combined_launch_config_param + 1
		          combined_launch_config_param_name(n_combined_launch_config_param,1) &
			  = nc_config_params_lst(i_table)
			  combined_launch_config_param_name(n_combined_launch_config_param,2) &
			  = 'META'  ! To indicate config is from Ascii META file
			  !write(msg,*) "DBG:>>> META launch name ",nc_config_params_lst(i_table); call wrtlog(sub,msg)
                          !write(msg,*) "DBG:>>> META launch val ", str_meta_tmp3; call wrtlog(sub,msg)
			  if(is_numeric(trim(str_meta_tmp3)))then
			     read(str_meta_tmp3, *) config_val_tmp
			     call convert_config_param (qc_config_params_lst(i_table), &
			     nc_config_params_lst(i_table), config_val_tmp)
			     combined_launch_config_param_value(n_combined_launch_config_param) = config_val_tmp
			  else
		             combined_launch_config_param_value(n_combined_launch_config_param) = DBLE(99999)
			  endif
	               endif
		    enddo
		 endif
         enddo
      elseif(launch_config_source_indicator == 0)then ! Cycle 0 for Argos, Cycle 1 for Iridium 
         if(trans_type == 'IRIDIUM')then ;cycnum=1;
         else; cycnum=0; endif
         conf_sampl_match_cnt=0;
         do ipqc =1,npqc
		   is_valid_config = .true.;
		   do i_table =1,cnt_config_table_skip_param
		      str_pqc_tmp1=pqcline(ipqc); str_pqc_tmp2=str_pqc_tmp1(1:40);
		      call to_upper(str_pqc_tmp2)
		      if (trim(str_pqc_tmp2(1:40)).eq.'NUMBER OF COLUMNS') then
			 is_valid_config=.false.;
			 exit;						! exit the loop, not the program
		      elseif (trim(str_pqc_tmp2(3:40)).eq.'COLUMN') then
			 is_valid_config=.false.
			 exit;						! exit the loop, not the program
		      elseif (trim(str_pqc_tmp2(4:40)).eq.'COLUMN') then
			 is_valid_config=.false.
			 exit;						! exit the loop, not the program
		      elseif (trim(str_pqc_tmp2(5:40)).eq.'COLUMN') then
			 is_valid_config=.false.
			 exit;						! exit the loop, not the program
		      elseif (trim(str_pqc_tmp2(6:40)).eq.'COLUMN') then
			 is_valid_config=.false.
			 exit;						! exit the loop, not the program
		      elseif (str_pqc_tmp2(1:20).eq.'====================') then
			 is_valid_config=.false.
			 exit;						! exit the loop, not the program
		      elseif (trim(str_pqc_tmp2).eq.trim(qc_config_skip_lst(i_table))) then
			 is_valid_config=.false.
			 exit;						! exit the loop, not the program
		      endif
		   enddo	
		
		   if(is_valid_config) then ! Adding config from  Cycle 0 
		      str_pqc_tmp1=pqcline(ipqc); str_pqc_tmp2=str_pqc_tmp1(1:40);
		      str_pqc_tmp3=trim(str_pqc_tmp1(41:));
		      do i_table=1,cnt_config_table_param
			 call to_upper(str_pqc_tmp2);
			 !write (msg, *) "DBG:>>>ABCDEF :",str_pqc_tmp2;  call wrtlog(sub,msg) 
			 if (trim(str_pqc_tmp2).eq.trim(qc_config_params_lst(i_table))) then 
                            if(trans_type == 'IRIDIUM' .and. trim(nc_config_params_lst(i_table))== &
                            'CONFIG_TransmissionRepetitionPeriod_seconds')exit !Note from Claudia: this config is not needed for "Iridium"
			    !Note: According to the email from Annie Wong, adding code below to extract
          		    !      values from "CONFOG SAMPLING SCHEME" for UW floats.
                            if(trim(str_pqc_tmp2) == 'CONFIG SAMPLING SCHEME (DBAR)')then
	                        conf_sampl_match_cnt=conf_sampl_match_cnt + 1;
	                        call extract_conf_sampling_scheme(0,conf_sampl_match_cnt,str_pqc_tmp3);
	                    endif
		!           Found a parameter matching the left coulmn from config table
		            n_combined_launch_config_param = n_combined_launch_config_param + 1
		            combined_launch_config_param_name(n_combined_launch_config_param,1) &
			    = nc_config_params_lst(i_table)
			    combined_launch_config_param_name(n_combined_launch_config_param,2) &
			    = 'CYC0'  ! To indicate config is from Cycle 0 File
			    !write(msg,*) "DBG:>>> Cyc0 launch config name ", nc_config_params_lst(i_table); call wrtlog(sub,msg)
                            !write(msg,*) "DBG:>>> Cyc0 launch config val ", str_pqc_tmp3; call wrtlog(sub,msg)
                          
			    if(is_numeric(trim(str_pqc_tmp3)))then
			       read(str_pqc_tmp3, *) config_val_tmp
			       call convert_config_param (qc_config_params_lst(i_table), &
			       nc_config_params_lst(i_table), config_val_tmp)
			       combined_launch_config_param_value(n_combined_launch_config_param) = config_val_tmp
			    else
		              combined_launch_config_param_value(n_combined_launch_config_param) = DBLE(99999)
			    endif
		         endif
		      enddo
		   endif
         enddo
      endif
		      
      do i_table=1,n_combined_launch_config_param
         config_param_name_exists = .false.
        !write (msg,*) combined_launch_config_param_name(i_table, 1) ; call wrtlog("prepare launch",msg);
	    !write (msg,*) combined_launch_config_param_name(i_table, 2) ; call wrtlog("prepare launch",msg);
	    !write (msg,*) combined_launch_config_param_value(i_table) ; call wrtlog("prepare launch",msg);
	    do i_launch_config = 1 , n_launch_config_param ! Checks if config param name is already part of the array
	      if(trim(launch_configuration_parameter_name(i_launch_config)) &
	        == trim(combined_launch_config_param_name(i_table,1)))then 
	          config_param_name_exists = .true.
		  if(launch_configuration_parameter_value(i_launch_config) /= &
		    combined_launch_config_param_value(i_table))then  ! if they have differnt values one of the files could be wrong
		    fil = trim(launch_configuration_parameter_name(i_launch_config));
		    write(wrnmsg,'("WARNING 185 Config-vals mismatch b/w Meta & Cyc(0-ARGOS,1-IRIDIUM) for ",a,": ",f8.3,"(META),", &
		    f8.3,"(CYC ",i3.3,")")')trim(fil),launch_configuration_parameter_value(i_launch_config), &
		    combined_launch_config_param_value(i_table),cycnum;
		    if(thiscyc<6)then;
			call warning(sub,185,wrnmsg); ! WARNING 185 WRN<6 cnfg vals mismatch ! WRN
		    else
			call lognote(sub,185,wrnmsg); ! WARNING 185 LGN<6 cnfg vals mismatch
		    endif
		  endif
		  exit
	       endif
	    enddo
	    if(.NOT. config_param_name_exists)then
	       n_launch_config_param = n_launch_config_param + 1
	       launch_configuration_parameter_name(n_launch_config_param)  = combined_launch_config_param_name(i_table, 1)
	       launch_configuration_parameter_value(n_launch_config_param) = combined_launch_config_param_value(i_table)
            elseif(trans_type == 'IRIDIUM')then
               launch_configuration_parameter_name(i_launch_config)  = combined_launch_config_param_name(i_table, 1)
               launch_configuration_parameter_value(i_launch_config) = combined_launch_config_param_value(i_table)
	    endif
      enddo 
      
      
      
      !-----------------------------------------------------------------------------------------------------------------------
     ! if(trans_type == 'IRIDIUM')then
	!      do i_cycle0_param = 1 , n_cycle0_config_param
         !         launch_configuration_parameter_name(i_cycle0_param)= cycle0_configuration_parameter_name(i_cycle0_param)
	!	  launch_configuration_parameter_value(i_cycle0_param) = cycle0_configuration_parameter_value(i_cycle0_param)
         !     enddo
	  !    n_launch_config_param = n_cycle0_config_param
      !else
	!      if(trans_type == 'ARGOS') then 
        !       do i_met = 1,nmet
	 !        str_meta_tmp1 = trim(metline(i_met)); 
	!	 str_meta_tmp2=trim(str_meta_tmp1(1:40));
	!	 call to_upper(str_meta_tmp2);
         !        str_meta_tmp3=trim(str_meta_tmp1(41:168));
	!	 do i_table=1,cnt_config_table_param
	!	   if (trim(str_meta_tmp2).eq.trim(qc_config_skip_lst(i_table))) then
         !              exit;  ! exit the loop, not the program
          !         endif
	!	   if (trim(str_meta_tmp2) .eq. trim(qc_config_params_lst(i_table)))then
	!	        n_launch_config_param = n_launch_config_param + 1
	!	        launch_configuration_parameter_name(n_launch_config_param) = nc_config_params_lst(i_table)
	!		read(str_meta_tmp3, *) config_val_tmp
	!	        call convert_config_param (qc_config_params_lst(i_table), &
	!		      nc_config_params_lst(i_table), config_val_tmp)
	!		launch_configuration_parameter_value(n_launch_config_param) = config_val_tmp  
	 !          endif
	!	 enddo
	 !     enddo
      !endif   
      !-----------------------------------------------------------------------------------------------------------------------

if(DEBUG)then; write(msg,'("==== EXTNC META = <<<<< Leave ",a," ret= ",i3)')sub,prepare_launch_config;call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,'("==== ",a," END ====================")')typ;call wrtdbg(sub,msg); endif


return;
end function prepare_launch_config
! ===========================================================================================================
