! ===========================================================================================================
! integer function qc04_position_on_land.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20180619 VH     formt/string problems (vargo)
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
!
! Function return values:
!  0 = PASS - Success, float is in the water
!  1 = FAIL - Failure, float is on land
! -1 = NOTD - cannot be determined
!

integer function qc04_position_on_land(caller)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller

character(len=14), parameter :: sub = 'QC04'
character(len=21) ,parameter :: QCTEST='Position On Land'
integer :: result;

integer :: depth;
real    :: rlat, rlon;
integer :: nbad, ngud
integer :: on_land		! local result of land_or_water
integer :: j			! In case problems with on_lad
integer :: num_pos_lines;
integer :: mxtflag; ! Maximum local test flag
! ===========================================================================================================
! executable statements

result = NOTD ! cannot be determined

if(DBGIO) call dbg_enter(sub)

if(DBLEV== 4)then; write(msg,'("LEV04 lat/lon to test: ",2f12.4)')prof_LATITUDE,prof_LONGITUDE; call wrtdbg(sub,msg); endif
if(caller == 'read_pqc')then;  num_pos_lines = ppos; else; num_pos_lines = npos;endif
if(num_pos_lines==0)then
    write(msg,'("There are no positins, the land/water determination cannot be done!")');
    call wrtlog(sub,msg); call wrttrc(sub,msg);
    go to 999;   ! Skip to THE_END (npos==- so positions)
endif

nbad = 0; ngud = 0;
if(DBLEV== 4)then; write(msg,'("LEV04 POSNQC npos: ",i2)')num_pos_lines; call wrtdbg(sub,msg); endif
do i=1,num_pos_lines; pos_onland(i) = '0';  enddo
do i=1,num_pos_lines
    if(DBLEV== 4)then;
	write(msg,'("LEV04 POSNQC pos_posiqc     i: ",i2," qc: ",a)')i,pos_posiqc(i); call wrtdbg(sub,msg);
	if(pos_posiqc(i) /= '1')then;
	    write(msg,'("LEV04 POSNQC NOT EQUAL TO 1 i: ",i2," qc: ",a)')i,pos_posiqc(i); call wrtdbg(sub,msg);
	    call wrtdbg(sub,msg);
	 endif
    endif
    if(pos_posiqc(i) /= '1') go to 33;	! skip the test, not a good position
    rlat = this_lat(i); rlon = this_lon(i);

    on_land = land_or_water(sub,rlat,rlon);

    if(DBLEV== 4)then;
	write(msg,*)'LEV04 <<<<< DEBUG from QC >>>>>';				call wrttrc(sub,msg);
	write(msg,*)'LEV04 ETOPO: on_land=',on_land,' REAL rlat,rlon=',rlat,rlon;	call wrttrc(sub,msg);
    endif
    if(i > MAXPOS)then
	write(errmsg(1),'("NON-FATAL ERROR 244 in ",a," profile: ",a)')trim(sub),trim(profile)
	write(errmsg(2),'("NON-FATAL ERROR 244 QC04 trapped index out of bounts LARGE in qc04 1st occurrence")');
	write(errmsg(3),'("NON-FATAL ERROR 244 QC04 value of is is: ",a)')trim(msg)
	call error_report(sub,244,3,errmsg); ! qc04 trapped index out of bounds LARGE
	do j=1,num_pos_lines; pos_onland(j) = '0'; enddo; return
    endif
    if(i < 1)then
	write(errmsg(1),'("NON-FATAL ERROR 245 in ",a," profile: ",a)')trim(sub),trim(profile)
	write(errmsg(2),'("NON-FATAL ERROR 245 QC04 trapped index out of bounts SMALL in qc04 1st occurrence")');
	write(errmsg(3),'("NON-FATAL ERROR 245 QC04 value of is is: ",a)')trim(msg)
	call error_report(sub,245,3,errmsg); ! qc04 trapped index out of bounds in SMALL
	do j=1,num_pos_lines; pos_onland(j) = '0'; enddo; return
    endif
    if(on_land == 0)then			! position is in the water
	pos_onland(i) = '1'			! NOT on land, so flag is 1 - profile is ok
	ngud = ngud + 1;
    elseif(on_land > 0)then			! position is on land - on_land = altitude above sea level
	pos_onland(i) = '4'			! on land, so flag is 4 - profile is bad
	! NOT NEESSARILY A BAD POSITION pos_posiqc(i)  = '4';		! A position on land is NOT a good position
	prof_on_land = .true.		! at lease one position IS on land
	nbad = nbad + 1;
    else					! TOPOGRAPHY database not found for negative on_lan1G/
	if(DEBUG)then; write(msg,*)'Error No TOPO_FILE exists.'; call wrtstd(sub,msg); endif
	go to 999;   ! Skip to THE_END (Noo TOPO file, cannot determine on_land status)
    endif
    33 continue;                        ! skip the test
enddo

if(nbad > 0)then
    result = FAIL;	! at least one position is on lnad
    write(msg,'("The float is on LAND!!!")');  call wrtstd(sub,msg);   ! Special case, report to STD if on land
elseif(ngud > 0)then	! no bad and at least one good
    result = PASS;	! all the positions are n the water
    write(msg,'("The float is in WATER!!!")'); call wrtlog(sub,msg);   ! Normal case, report to TRC,LOG
else
    result = NOTD;	! No positions survived previous tests
    write(msg,'("No posiitons survived previsou tests, the land/water determination cannot be done!")');

endif
if(DBLEV== 4)then; write(msg,'("LEV04 QC04 the result is: ",i2," nbad/ngud: ",2i3)')result,nbad,ngud; call wrtdbg(sub,msg); endif
999 continue ! This is THE_END

mxtflag = 0; if(result == PASS)then; mxtflag = 1; elseif(result == FAIL)then; mxtflag = 4; endif
if(DEBUG) write(DBG,'("QCTEST_FLAG QC04 result: ",i2," mxtflag: ",i2," nprf: ",i2)')result,mxtflag,nprf
do i=1,nprf; QCTEST_FLAG(i,4) = mxtflag; enddo

if(DBGIO) call dbg_leave(sub,result);

qc04_position_on_land = result;
if(result == PASS)then; result = 1; elseif(result == FAIL)then; result = 4; else; result = 0; endif

return
end function qc04_position_on_land
! ===========================================================================================================
