! ===========================================================================================================
! integer function qc11_gradient_test.f90
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
!
! Function return values:
!  0 = PASS - Success, all positions passed
!  1 = FAIL - Failure, some or all positions failed
! -1 = NOTD - cannot be determined
!

integer function qc11_gradient_test(caller,info)

use communication                       ! module
use data_arrays                         ! module

implicit none

! ===========================================================================================================
! varialbe/array declarations

character(*), intent( in ) :: caller
integer,      intent( in ) :: info(3)

character(len=4), parameter :: sub = 'QC11'
character(len=21) ,parameter :: QCTEST='Gradient Test'
integer :: result;

integer :: ifl, itype, beg, end;
real :: profdata_pres(MAXLEV),profdata_tmp(MAXLEV)
integer :: ilev_vector_tmp(MAXLEV);
integer :: proftot_tmp

integer :: ngood,nbad,nskip;
real, parameter :: range1(4) = (/   0.0, 9.0,  1.5,  50.0 /)
real, parameter :: range2(4) = (/   0.0, 3.0,  0.5,  25.0 /)

integer :: pressure_toCompare = 500
real :: tval; ! test value

! ===========================================================================================================
! executable statements

ifl = vertgrad_ncol; 

result = NOTD ! cannot be determined

itype=info(1); beg=info(2); end=info(3); ! run characteristics

if(DBGIO) call dbg_enter_type(sub,parmname(itype))

do idat=beg,end
    if(hasDATA(idat))then                                                                          ! if hasDATA
        if(proftot(idat) > 0)then                                                                 ! if ntot>0
            ngood = 0; nbad = 0; nskip = 0;
            call fill_profvect_tmp_2param(proftot_tmp,profdata_tmp,profdata_pres,idat,pbud(idat),ilev_vector_tmp)
	    if(proftot_tmp < 3)then
		write(msg,'("profvect filter (2p) finds only ",i2," good point(s)")')proftot_tmp;
		call wrtlog(sub,msg); go to 33; !Skip to end of loop
	    endif
	    ! The first and last levels OF THE TEMPORARY ARRAY are not tested
	    testflag(idat,ilev_vector_tmp(1),ifl) = NOTF; testflag(idat,ilev_vector_tmp(proftot_tmp),ifl) = NOTF;
            do ilev=2,proftot_tmp-1
                    tval = abs(profdata_tmp(ilev)-((profdata_tmp(ilev+1)+profdata_tmp(ilev-1))/2) ) 
                    if(tval > range1(itype) .AND. profdata_pres(ilev) < pressure_toCompare  )then
                        testflag(idat,ilev_vector_tmp(ilev),ifl) = BAD; nbad = nbad + 1;
                        if(DBLEV==11) call report_ngnbns(sub,idat,ilev,ifl,ngood,nbad,nskip,"bad1")
                    elseif(tval > range2(itype) .AND.  profdata_pres(ilev) >= pressure_toCompare )then
                        testflag(idat,ilev_vector_tmp(ilev),ifl) = BAD; nbad = nbad + 1;
                        if(DBLEV==11) call report_ngnbns(sub,idat,ilev,ifl,ngood,nbad,nskip,"bad2")
                    else
                        testflag(idat,ilev_vector_tmp(ilev),ifl) = GOOD; ngood = ngood + 1;
			  if(DBLEV==11) call report_ngnbns(sub,idat,ilev,ifl,ngood,nbad,nskip,"gud")
                    endif
            enddo
	    33 continue;        ! Skip to end of loop HERE
        endif ! if(proftot(idat) == 0)                                                          ! if ntot>0
        call evaluate_flags(sub,QCTEST,datname(idat),idat,ifl,beg,result); maxresult=max(maxresult,result);
    endif                                                                                       ! if hasDATA
enddo                                                                                           ! idat=beg,end

! The result returned is the max flag of those evaluated
qc11_gradient_test = maxresult;
if(DBGIO) call dbg_leave_type(sub,parmname(itype),result);

return
end function qc11_gradient_test
! ===========================================================================================================
