! ===========================================================================================================
! integer function read_trajtable
!
! -----------------------------------------------------------------------------------------------------------
!
! Revision history
! date     who    what
! 20170712 all    Release of qcplus v1.1 - many changes to all, some documented/tracked, some not
! 20160511 all    Initial release of qcplus v1.1.0
!
! ===========================================================================================================
! Notes
!
! Function return values:
!  1 = Success, profile passed
! -1 = Failure, profile failed
!  0 = Unable to determine
!
! ===========================================================================================================

integer function read_trajtable(caller)

use communication                       ! module
use data_arrays                         ! module
use nctraj_arrays                       ! module

implicit none
character(*), intent( in ) :: caller


integer :: result = 0; ! number of table values read
character(len=16),parameter :: sub = 'read_trajtable  '
character(len=15),parameter :: typ = 'NCTRAJ RDTTRAJ'
 
!--------------------------------- Variable definitions -----------------------------------------
character(len=168) :: line ! One line read from file
integer, parameter :: TRAJ_FILE_UNIT = 10  ! Handler 
integer :: status, ios, lnblnk , idx
integer, parameter :: IO_EOF = -1  ! End of File 
character(len=40) :: one_traj_param

logical :: table_exists;

! ===========================================================================================================
! executable statements

read_trajtable=FAILURE;						! Assume failure
! Read traj table 

if(DEBUG)then; write(msg,'("==== ",a," BEG ====================")')typ;call wrtdbg(sub,msg); endif

result = NOTD; ! "can not be determined."

inp_open=open_old_file_nf(TRAJ_FILE_UNIT,pathname_traj_table,"TRAJ","TABLE");
if(.not. inp_open)then;
    write(msg,'("READ_TRAJTABLE Failed to open talbe fie: ",a)')trim(pathname_traj_table); call wrttrc(sub,msg);
    goto 960
endif
cnt_traj_table_param = 0
read(TRAJ_FILE_UNIT,'(168a)',end=101,err=961,iostat=ios) line
do while (ios==IO_OK)
   cnt_traj_table_param = cnt_traj_table_param + 1
   qc_traj_params_lst(cnt_traj_table_param)= line(1:40)
   nc_traj_params_lst(cnt_traj_table_param)= line(41:168)
   read(TRAJ_FILE_UNIT,'(168a)',end=101,err=961,iostat=ios) line
end do
101 continue
if (ios/=IO_EOF) goto 961 ! if there is any error, XXX needed?
close(unit=TRAJ_FILE_UNIT);

! Read traj table skip
inp_open=open_old_file_nf(TRAJ_FILE_UNIT,pathname_traj_table_skip,"TRAJ","TABLE_SKIP");
if(.not. inp_open)then
    write(msg,'("READ_TRAJTABLE Failed to open talbe fie: ",a)')trim(pathname_traj_table_skip); call wrttrc(sub,msg);
   goto 962
endif
cnt_traj_table_skip_param = 0
read(TRAJ_FILE_UNIT,'(a)',end=102,err=963,iostat=ios) line
do while (ios==IO_OK)
   cnt_traj_table_skip_param = cnt_traj_table_skip_param + 1
   read(line(1:40),'(a)') qc_traj_skip_lst(cnt_traj_table_skip_param)
   read(TRAJ_FILE_UNIT,'(a)',end=102,err=963,iostat=ios) line
end do
102 continue
if (ios/=IO_EOF) goto 963 ! if there is any error, XXX needed?
close(unit=TRAJ_FILE_UNIT);



! Read traj table with MC
inp_open=open_old_file_nf(TRAJ_FILE_UNIT, pathname_traj_mc_table,"TRAJ","TABLE_MC");
if(.not. inp_open)then
    write(msg,'("READ_TRAJTABLE Failed to open talbe fie: ",a)')trim(pathname_traj_mc_table); call wrttrc(sub,msg);
    goto 964
endif
cnt_traj_mc_table_param = 0
read(TRAJ_FILE_UNIT,'(a)',end=103,err=965,iostat=ios) line
do while (ios==IO_OK)
   cnt_traj_mc_table_param = cnt_traj_mc_table_param + 1
   read(line(1:40),'(a)') qc_traj_mc_params_lst(cnt_traj_mc_table_param)
   nc_traj_mc_params_lst(cnt_traj_mc_table_param)= line(41:168)
   read(TRAJ_FILE_UNIT,'(a)',end=103,err=965,iostat=ios) line
end do
103 continue
if (ios/=IO_EOF) goto 965 ! if there is any error
close(unit=TRAJ_FILE_UNIT);


read_trajtable=SUCCESS						! Signal SUCCESS
go to 999; ! Skip to THE_END

960 continue
    read_trajtable=FAILURE
    write(errmsg(1),'("NON-FATAL ERROR 285 opening the TRAJ_TABLE file")');
    write(errmsg(2),'("NON-FATAL ERROR 285 file: ",a)') trim(pathname_traj_table);
    write(errmsg(3),'("NON-FATAL ERROR 285 desc: TABLE")')
    call error_report(sub,285,3,errmsg); ! read_trajtable error opening TRAJ_TABLE
    go to 999; ! Skip to THE_END
961 continue
    read_trajtable=FAILURE
    write(errmsg(1),'("NON-FATAL ERROR 286 reading TRAJ_TABLE")');
    write(errmsg(2),'("NON-FATAL ERROR 286 read reported error ios: ",i2)')ios;
    close(unit=TRAJ_FILE_UNIT);
    call error_report(sub,286,2,errmsg); ! read_trajtable error reading TRAJ_TABLE
    go to 999; ! Skip to THE_END
962 continue
    read_trajtable=FAILURE
    write(errmsg(1),'("NON-FATAL ERROR 287 opening the TRAJ_TABLE_SKIP file")');
    write(errmsg(2),'("NON-FATAL ERROR 287 file: ",a)') trim(pathname_traj_table_skip);
    write(errmsg(3),'("NON-FATAL ERROR 287 desc: TABLE_SKIP")')
    call error_report(sub,287,3,errmsg); ! read_trajtable error opening TRAJ_TABLE_SKIP
    go to 999; ! Skip to THE_END
963 continue
    read_trajtable=FAILURE
    write(errmsg(1),'("NON-FATAL ERROR 288 reading TRAJ_TABLE_SKIP")');
    write(errmsg(2),'("NON-FATAL ERROR 288 read reported error ios: ",i2)')ios;
    close(unit=TRAJ_FILE_UNIT);
    call error_report(sub,288,2,errmsg); ! read_trajtable error reading TRAJ_TABLE_SKIP
964 continue
    read_trajtable=FAILURE
    write(errmsg(1),'("NON-FATAL ERROR 289 opening the TRAJ_MC_TABLE file")');
    write(errmsg(2),'("NON-FATAL ERROR 289 file: ",a)') trim(pathname_traj_mc_table);
    write(errmsg(3),'("NON-FATAL ERROR 289 desc: TRAJ_MC_TABLE")')
    call error_report(sub,289,3,errmsg); ! read_trajtable error opening TRAJ_MC_TABLE
    go to 999; ! Skip to THE_END
965 continue
    read_trajtable=FAILURE
    write(errmsg(1),'("NON-FATAL ERROR 290 reading TRAJ_MC_TABLE")');
    write(errmsg(2),'("NON-FATAL ERROR 290 read reported error ios: ",i2)')ios;
    close(unit=TRAJ_FILE_UNIT);
    call error_report(sub,290,2,errmsg); ! read_trajtable  error reading TRAJ_MC_TABLE
    
999 continue ! This is THE_END

if(DEBUG)then; write(msg,'("NCTRAJ Number of traj params: ",i4)')cnt_traj_table_param; call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,'("NCTRAJ Number of skip params: ",i4)')cnt_traj_table_skip_param; call wrtdbg(sub,msg); endif

if(DEBUG)then; write(msg,'("==== RDTNC TRAJ <<<<< Leave ",a," ret= ",i3)')sub,read_trajtable;call wrtdbg(sub,msg); endif
if(DEBUG)then; write(msg,'("==== ",a," END ====================")')typ;call wrtdbg(sub,msg); endif

return
end function read_trajtable
